@@ -4,21 +4,41 @@
 #
 # Copyright (C) 2018, James R. Barlow (https://github.com/jbarlow83/)
 
+import logging
 import re
 import sys
-from collections import namedtuple
-from collections.abc import MutableMapping
+from abc import ABC, abstractstaticmethod
 from datetime import datetime
 from functools import wraps
 from io import BytesIO
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Callable,
+    Dict,
+    List,
+    NamedTuple,
+    Optional,
+    Set,
+    Type,
+    Union,
+)
 from warnings import warn
 
 from lxml import etree
-from lxml.etree import parse, QName, XMLSyntaxError
+from lxml.etree import QName, XMLParser, XMLSyntaxError, parse
 
-from .. import Name, PdfError, Stream, String
+from .. import Name, Stream, String
 from .. import __version__ as pikepdf_version
 
+if sys.version_info < (3, 9):  # pragma: no cover
+    from typing import Iterable, MutableMapping
+else:
+    from collections.abc import Iterable, MutableMapping
+
+if TYPE_CHECKING:  # pragma: no cover
+    from pikepdf import Pdf
+
 XMP_NS_DC = "http://purl.org/dc/elements/1.1/"
 XMP_NS_PDF = "http://ns.adobe.com/pdf/1.3/"
 XMP_NS_PDFA_ID = "http://www.aiim.org/pdfa/ns/id/"
@@ -62,7 +82,20 @@ XMP_EMPTY = b"""<x:xmpmeta xmlns:x="adobe:ns:meta/" x:xmptk="pikepdf">
 
 XPACKET_END = b"""\n<?xpacket end="w"?>\n"""
 
-XmpContainer = namedtuple('XmpContainer', ['rdf_type', 'py_type', 'insert_fn'])
+
+class XmpContainer(NamedTuple):
+    rdf_type: str
+    py_type: Type
+    insert_fn: Callable
+
+
+log = logging.getLogger(__name__)
+
+
+class NeverRaise(Exception):
+    """An exception that is never raised"""
+
+    pass  # pylint: disable=unnecessary-pass
 
 
 class AltList(list):
@@ -87,9 +120,28 @@ LANG_ALTS = frozenset(
 # These are the illegal characters in XML 1.0. (XML 1.1 is a bit more permissive,
 # but we'll be strict to ensure wider compatibility.)
 re_xml_illegal_chars = re.compile(
-    r"(?u)[^\x09\x0A\x0D\x20-\uD7FF\uE000-\uFFFD\u10000-\u10FFFF]"
+    r"(?u)[^\x09\x0A\x0D\x20-\U0000D7FF\U0000E000-\U0000FFFD\U00010000-\U0010FFFF]"
 )
-re_xml_illegal_bytes = re.compile(br"[^\x09\x0A\x0D\x20-\xFF]|&#0;")
+re_xml_illegal_bytes = re.compile(
+    br"[^\x09\x0A\x0D\x20-\xFF]|&#0;"
+    # br"&#(?:[0-9]|0[0-9]|1[0-9]|2[0-9]|3[0-1]|x[0-9A-Fa-f]|x0[0-9A-Fa-f]|x1[0-9A-Fa-f]);"
+)
+
+
+def _clean(s: Union[str, Iterable[str]], joiner: str = '; ') -> str:
+    """Ensure an object can safely be inserted in a XML tag body.
+
+    If we still have a non-str object at this point, the best option is to
+    join it, because it's apparently calling for a new node in a place that
+    isn't allowed in the spec or not supported.
+    """
+    if not isinstance(s, str) and isinstance(s, Iterable):
+        warn(f"Merging elements of {s}")
+        if isinstance(s, Set):
+            s = joiner.join(sorted(s))
+        else:
+            s = joiner.join(s)
+    return re_xml_illegal_chars.sub('', s)
 
 
 def encode_pdf_date(d: datetime) -> str:
@@ -109,17 +161,19 @@ def encode_pdf_date(d: datetime) -> str:
     considered unknown. Regardless of whether or not GMT
     information is specified, the remainder of the string should specify
     the local time.
+
+    'D:' is required in PDF/A, so we always add it.
     """
 
     # The formatting of %Y is not consistent as described in
     # https://bugs.python.org/issue13305 and underspecification in libc.
     # So explicitly format the year with leading zeros
-    s = "{:04d}".format(d.year)
+    s = f"D:{d.year:04d}"
     s += d.strftime(r'%m%d%H%M%S')
     tz = d.strftime('%z')
     if tz:
         sign, tz_hours, tz_mins = tz[0], tz[1:3], tz[3:5]
-        s += "{}{}'{}'".format(sign, tz_hours, tz_mins)
+        s += f"{sign}{tz_hours}'{tz_mins}'"
     return s
 
 
@@ -147,7 +201,17 @@ def decode_pdf_date(s: str) -> datetime:
         return datetime.strptime(s, r'%Y%m%d%H%M%S')
 
 
-class AuthorConverter:
+class Converter(ABC):
+    @abstractstaticmethod
+    def xmp_from_docinfo(docinfo_val: Optional[str]) -> Any:
+        "Derive XMP metadata from a DocumentInfo string"
+
+    @abstractstaticmethod
+    def docinfo_from_xmp(xmp_val: Any) -> Optional[str]:
+        "Derive a DocumentInfo value from equivalent XMP metadata"
+
+
+class AuthorConverter(Converter):
     @staticmethod
     def xmp_from_docinfo(docinfo_val):
         return [docinfo_val]
@@ -156,27 +220,44 @@ class AuthorConverter:
     def docinfo_from_xmp(xmp_val):
         if isinstance(xmp_val, str):
             return xmp_val
+        elif xmp_val is None or xmp_val == [None]:
+            return None
         else:
             return '; '.join(xmp_val)
 
 
-if sys.version_info < (3, 7):
+def _fromisoformat_py36(datestr: str) -> datetime:
+    """Backported equivalent of datetime.fromisoformat
 
-    def fromisoformat(datestr):
-        # strptime %z can't parse a timezone with punctuation
-        if re.search(r'[+-]\d{2}[-:]\d{2}$', datestr):
-            datestr = datestr[:-3] + datestr[-2:]
+    Can remove whenever we drop Python 3.6 support.
+    """
+    # strptime %z can't parse a timezone with punctuation
+    if re.search(r'[+-]\d{2}[-:]\d{2}$', datestr):
+        datestr = datestr[:-3] + datestr[-2:]
+    formats = [
+        "%Y-%m-%dT%H:%M:%S%z",
+        "%Y-%m-%dT%H:%M:%S",
+        "%Y-%m-%dT%H:%M:%S.%f%z",
+        "%Y-%m-%dT%H:%M:%S.%f",
+    ]
+    for fmt in formats:
         try:
-            return datetime.strptime(datestr, "%Y-%m-%dT%H:%M:%S%z")
+            return datetime.strptime(datestr, fmt)
         except ValueError:
-            return datetime.strptime(datestr, "%Y-%m-%dT%H:%M:%S")
+            continue
+    raise ValueError(
+        f"Could not parse ISO date: {datestr}. Try Python 3.7+, which "
+        "has better support for ISO dates."
+    )
 
 
+if sys.version_info < (3, 7):
+    fromisoformat = _fromisoformat_py36
 else:
     fromisoformat = datetime.fromisoformat
 
 
-class DateConverter:
+class DateConverter(Converter):
     @staticmethod
     def xmp_from_docinfo(docinfo_val):
         if docinfo_val == '':
@@ -191,6 +272,13 @@ class DateConverter:
         return encode_pdf_date(dateobj)
 
 
+class DocinfoMapping(NamedTuple):
+    ns: str
+    key: str
+    name: Name
+    converter: Optional[Type[Converter]]
+
+
 def ensure_loaded(fn):
     @wraps(fn)
     def wrapper(self, *args, **kwargs):
@@ -232,28 +320,37 @@ class PdfMetadata(MutableMapping):
         :meth:`pikepdf.Pdf.open_metadata`
     """
 
-    DOCINFO_MAPPING = [
-        (XMP_NS_DC, 'creator', Name.Author, AuthorConverter),
-        (XMP_NS_DC, 'description', Name.Subject, None),
-        (XMP_NS_DC, 'title', Name.Title, None),
-        (XMP_NS_PDF, 'Keywords', Name.Keywords, None),
-        (XMP_NS_PDF, 'Producer', Name.Producer, None),
-        (XMP_NS_XMP, 'CreateDate', Name.CreationDate, DateConverter),
-        (XMP_NS_XMP, 'CreatorTool', Name.Creator, None),
-        (XMP_NS_XMP, 'ModifyDate', Name.ModDate, DateConverter),
+    DOCINFO_MAPPING: List[DocinfoMapping] = [
+        DocinfoMapping(XMP_NS_DC, 'creator', Name.Author, AuthorConverter),
+        DocinfoMapping(XMP_NS_DC, 'description', Name.Subject, None),
+        DocinfoMapping(XMP_NS_DC, 'title', Name.Title, None),
+        DocinfoMapping(XMP_NS_PDF, 'Keywords', Name.Keywords, None),
+        DocinfoMapping(XMP_NS_PDF, 'Producer', Name.Producer, None),
+        DocinfoMapping(XMP_NS_XMP, 'CreateDate', Name.CreationDate, DateConverter),
+        DocinfoMapping(XMP_NS_XMP, 'CreatorTool', Name.Creator, None),
+        DocinfoMapping(XMP_NS_XMP, 'ModifyDate', Name.ModDate, DateConverter),
     ]
 
-    NS = {prefix: uri for uri, prefix in DEFAULT_NAMESPACES}
-    REVERSE_NS = {uri: prefix for uri, prefix in DEFAULT_NAMESPACES}
+    NS: Dict[str, str] = {prefix: uri for uri, prefix in DEFAULT_NAMESPACES}
+    REVERSE_NS: Dict[str, str] = {uri: prefix for uri, prefix in DEFAULT_NAMESPACES}
 
-    def __init__(self, pdf, pikepdf_mark=True, sync_docinfo=True):
+    def __init__(
+        self,
+        pdf: 'Pdf',
+        pikepdf_mark: bool = True,
+        sync_docinfo: bool = True,
+        overwrite_invalid_xml: bool = True,
+    ):
         self._pdf = pdf
         self._xmp = None
         self.mark = pikepdf_mark
         self.sync_docinfo = sync_docinfo
         self._updating = False
+        self.overwrite_invalid_xml = overwrite_invalid_xml
 
-    def load_from_docinfo(self, docinfo, delete_missing=False, raise_failure=False):
+    def load_from_docinfo(
+        self, docinfo, delete_missing: bool = False, raise_failure: bool = False
+    ) -> None:
         """Populate the XMP metadata object with DocumentInfo
 
         Arguments:
@@ -267,6 +364,13 @@ class PdfMetadata(MutableMapping):
         approximately equivalent to certain XMP records. This method copies
         those entries into the XMP metadata.
         """
+
+        def warn_or_raise(msg, e=None):
+            if raise_failure:
+                raise ValueError(msg) from e
+            else:
+                warn(msg)
+
         for uri, shortkey, docinfo_name, converter in self.DOCINFO_MAPPING:
             qname = QName(uri, shortkey)
             # docinfo might be a dict or pikepdf.Dictionary, so lookup keys
@@ -282,54 +386,77 @@ class PdfMetadata(MutableMapping):
                     val = converter.xmp_from_docinfo(val)
                 if not val:
                     continue
-                self[qname] = val
-            except (ValueError, AttributeError) as e:
-                msg = "The metadata field {} could not be copied to XMP".format(
-                    docinfo_name
+                self._setitem(qname, val, True)
+            except (ValueError, AttributeError, NotImplementedError) as e:
+                warn_or_raise(
+                    f"The metadata field {docinfo_name} could not be copied to XMP", e
                 )
-                if raise_failure:
-                    raise ValueError(msg) from e
-                else:
-                    warn(msg)
+        valid_docinfo_names = set(
+            str(docinfo_name) for _, _, docinfo_name, _ in self.DOCINFO_MAPPING
+        )
+        extra_docinfo_names = set(str(k) for k in docinfo.keys()) - valid_docinfo_names
+        for extra in extra_docinfo_names:
+            warn_or_raise(
+                f"The metadata field {extra} with value '{repr(docinfo.get(extra))}' "
+                "has no XMP equivalent, so it was discarded",
+            )
 
-    def _load(self):
+    def _load(self) -> None:
         try:
             data = self._pdf.Root.Metadata.read_bytes()
         except AttributeError:
             data = XMP_EMPTY
         self._load_from(data)
 
-    def _load_from(self, data):
-        try:
-            self._xmp = parse(BytesIO(data))
-        except XMLSyntaxError:
-            data = re_xml_illegal_bytes.sub(b'', data)
+    def _load_from(self, data: bytes) -> None:
+        if data.strip() == b'':
+            data = XMP_EMPTY  # on some platforms lxml chokes on empty documents
+
+        def basic_parser(xml):
+            return parse(BytesIO(xml))
+
+        def strip_illegal_bytes_parser(xml):
+            return parse(BytesIO(re_xml_illegal_bytes.sub(b'', xml)))
+
+        def recovery_parser(xml):
+            parser = XMLParser(recover=True)
+            return parse(BytesIO(xml), parser)
+
+        def replace_with_empty_xmp(_xml=None):
+            log.warning("Error occurred parsing XMP, replacing with empty XMP.")
+            return basic_parser(XMP_EMPTY)
+
+        if self.overwrite_invalid_xml:
+            parsers: Iterable[Callable] = [
+                basic_parser,
+                strip_illegal_bytes_parser,
+                recovery_parser,
+                replace_with_empty_xmp,
+            ]
+        else:
+            parsers = [basic_parser]
+
+        for parser in parsers:
             try:
-                self._xmp = parse(BytesIO(data))
-            except XMLSyntaxError as e:
+                self._xmp = parser(data)
+            except (XMLSyntaxError if self.overwrite_invalid_xml else NeverRaise) as e:
                 if str(e).startswith("Start tag expected, '<' not found") or str(
                     e
                 ).startswith("Document is empty"):
-                    # This is usually triggered by processing instructions
-                    # in another otherwise empty document, or empty documents,
-                    # which we consider safe to coerce to a well-formed
-                    # XMP. For harder cases like truncated XMP, we want to
-                    # raise the exception so that someone is alerted.
-                    self._xmp = parse(BytesIO(XMP_EMPTY))
-                else:
-                    raise PdfError() from e
-        pis = self._xmp.xpath('/processing-instruction()')
-        for pi in pis:
-            etree.strip_tags(self._xmp, pi.tag)
-        try:
-            self._get_rdf_root()
-        except ValueError:
-            if self._xmp.find('.', self.NS).tag == '{adobe:ns:meta/}xmpmeta':
-                # Looks like: <x:xmpmeta></x:xmpmeta>, so reload with template
-                # that includes <rdf:RDF>
-                return self._load_from(XMP_EMPTY)
+                    self._xmp = replace_with_empty_xmp()
+                    break
             else:
-                raise  # Probably not XMP
+                break
+
+        try:
+            pis = self._xmp.xpath('/processing-instruction()')
+            for pi in pis:
+                etree.strip_tags(self._xmp, pi.tag)
+            self._get_rdf_root()
+        except (Exception if self.overwrite_invalid_xml else NeverRaise) as e:
+            log.warning("Error occurred parsing XMP", exc_info=e)
+            self._xmp = replace_with_empty_xmp()
+        return
 
     @ensure_loaded
     def __enter__(self):
@@ -350,7 +477,8 @@ class PdfMetadata(MutableMapping):
         The standard mapping is described here:
             https://www.pdfa.org/pdfa-metadata-xmp-rdf-dublin-core/
         """
-        self._pdf.docinfo  # Touch object to ensure it exists
+        # Touch object to ensure it exists
+        self._pdf.docinfo  # pylint: disable=pointless-statement
         for uri, element, docinfo_name, converter in self.DOCINFO_MAPPING:
             qname = QName(uri, element)
             try:
@@ -364,16 +492,20 @@ class PdfMetadata(MutableMapping):
                     value = converter.docinfo_from_xmp(value)
                 except ValueError:
                     warn(
-                        "The DocumentInfo field {} could not be updated from XMP".format(
-                            docinfo_name
-                        )
+                        f"The DocumentInfo field {docinfo_name} could not be "
+                        "updated from XMP"
                     )
                     value = None
+                except Exception as e:
+                    raise ValueError(
+                        "An error occurred while updating DocumentInfo field "
+                        f"{docinfo_name} from XMP {qname} with value {value}"
+                    ) from e
             if value is None:
                 if docinfo_name in self._pdf.docinfo:
                     del self._pdf.docinfo[docinfo_name]
                 continue
-            value = re_xml_illegal_chars.sub('', value)
+            value = _clean(value)
             try:
                 # Try to save pure ASCII
                 self._pdf.docinfo[docinfo_name] = value.encode('ascii')
@@ -398,8 +530,17 @@ class PdfMetadata(MutableMapping):
         Depending how we are initialized, leave our metadata mark and producer.
         """
         if self.mark:
-            self[QName(XMP_NS_XMP, 'MetadataDate')] = datetime.now().isoformat()
-            self[QName(XMP_NS_PDF, 'Producer')] = 'pikepdf ' + pikepdf_version
+            # We were asked to mark the file as being edited by pikepdf
+            self._setitem(
+                QName(XMP_NS_XMP, 'MetadataDate'),
+                datetime.now(datetime.utcnow().astimezone().tzinfo).isoformat(),
+                applying_mark=True,
+            )
+            self._setitem(
+                QName(XMP_NS_PDF, 'Producer'),
+                'pikepdf ' + pikepdf_version,
+                applying_mark=True,
+            )
         xml = self._get_xml_bytes()
         self._pdf.Root.Metadata = Stream(self._pdf, xml)
         self._pdf.Root.Metadata[Name.Type] = Name.Metadata
@@ -407,22 +548,23 @@ class PdfMetadata(MutableMapping):
         if self.sync_docinfo:
             self._update_docinfo()
 
-    def _qname(self, name):
+    @classmethod
+    def _qname(cls, name: Union[QName, str]) -> str:
         """Convert name to an XML QName
 
         e.g. pdf:Producer -> {http://ns.adobe.com/pdf/1.3/}Producer
         """
         if isinstance(name, QName):
-            return name
+            return str(name)
         if not isinstance(name, str):
-            raise TypeError("{} must be str".format(name))
+            raise TypeError(f"{name} must be str")
         if name == '':
             return name
         if name.startswith('{'):
             return name
         prefix, tag = name.split(':', maxsplit=1)
-        uri = self.NS[prefix]
-        return QName(uri, tag)
+        uri = cls.NS[prefix]
+        return str(QName(uri, tag))
 
     def _prefix_from_uri(self, uriname):
         """Given a fully qualified XML name, find a prefix
@@ -448,7 +590,7 @@ class PdfMetadata(MutableMapping):
                 return ''
 
         for xmlcontainer, container, insertfn in XMP_CONTAINERS:
-            items = node.find('rdf:{}'.format(xmlcontainer), self.NS)
+            items = node.find(f'rdf:{xmlcontainer}', self.NS)
             if items is None:
                 continue
             result = container()
@@ -465,7 +607,7 @@ class PdfMetadata(MutableMapping):
                 raise ValueError("Metadata seems to be XML but not XMP")
         return rdf
 
-    def _get_elements(self, name=''):
+    def _get_elements(self, name: Union[str, QName] = ''):
         """Get elements from XMP
 
         Core routine to find elements matching name within the XMP and yield
@@ -481,7 +623,7 @@ class PdfMetadata(MutableMapping):
         the elements. We ignore qualifiers.
 
         Args:
-            name (str): a prefixed name or QName to look for within the
+            name: a prefixed name or QName to look for within the
                 data section of the XMP; looks for all data keys if omitted
 
         Yields:
@@ -509,18 +651,18 @@ class PdfMetadata(MutableMapping):
         yield from (v[2] for v in self._get_elements(name))
 
     @ensure_loaded
-    def __contains__(self, key):
+    def __contains__(self, key: Union[str, QName]):
         try:
             return any(self._get_element_values(key))
         except KeyError:
             return False
 
     @ensure_loaded
-    def __getitem__(self, key):
+    def __getitem__(self, key: Union[str, QName]):
         try:
             return next(self._get_element_values(key))
         except StopIteration:
-            raise KeyError(key)
+            raise KeyError(key) from None
 
     @ensure_loaded
     def __iter__(self):
@@ -534,34 +676,63 @@ class PdfMetadata(MutableMapping):
     def __len__(self):
         return len(list(iter(self)))
 
-    @ensure_loaded
-    def __setitem__(self, key, val):
+    def _setitem(
+        self,
+        key: Union[str, QName],
+        val: Union[Set[str], List[str], str],
+        applying_mark: bool = False,
+    ):
         if not self._updating:
             raise RuntimeError("Metadata not opened for editing, use with block")
 
-        def clean(s):
-            return re_xml_illegal_chars.sub('', s)
+        qkey = self._qname(key)
+        if (
+            self.mark
+            and not applying_mark
+            and qkey
+            in (
+                self._qname('xmp:MetadataDate'),
+                self._qname('pdf:Producer'),
+            )
+        ):
+            # Complain if user writes self[pdf:Producer] = ... and because it will
+            # be overwritten on save, unless self._updating_mark, in which case
+            # the action was initiated internally
+            log.warning(
+                f"Update to {key} will be overwritten because metadata was opened "
+                "with set_pikepdf_as_editor=True"
+            )
+        if isinstance(val, str) and qkey in (self._qname('dc:creator')):
+            log.error(f"{key} should be set to a list of strings")
 
-        def add_array(node, items):
+        def add_array(node, items: Iterable):
             rdf_type = next(
                 c.rdf_type for c in XMP_CONTAINERS if isinstance(items, c.py_type)
             )
-            seq = etree.SubElement(node, QName(XMP_NS_RDF, rdf_type))
+            seq = etree.SubElement(node, str(QName(XMP_NS_RDF, rdf_type)))
+            tag_attrib: Optional[Dict[str, str]] = None
             if rdf_type == 'Alt':
-                attrib = {QName(XMP_NS_XML, 'lang'): 'x-default'}
-            else:
-                attrib = None
+                tag_attrib = {str(QName(XMP_NS_XML, 'lang')): 'x-default'}
             for item in items:
-                el = etree.SubElement(seq, QName(XMP_NS_RDF, 'li'), attrib=attrib)
-                el.text = clean(item)
+                el = etree.SubElement(
+                    seq, str(QName(XMP_NS_RDF, 'li')), attrib=tag_attrib
+                )
+                el.text = _clean(item)
 
         try:
             # Locate existing node to replace
             node, attrib, _oldval, _parent = next(self._get_elements(key))
             if attrib:
                 if not isinstance(val, str):
-                    raise TypeError(val)
-                node.set(attrib, clean(val))
+                    if qkey == self._qname('dc:creator'):
+                        # dc:creator incorrectly created as an attribute - we're
+                        # replacing it anyway, so remove the old one
+                        del node.attrib[qkey]
+                        add_array(node, _clean(val))
+                    else:
+                        raise TypeError(f"Setting {key} to {val} with type {type(val)}")
+                else:
+                    node.set(attrib, _clean(val))
             elif isinstance(val, (list, set)):
                 for child in node.findall('*'):
                     node.remove(child)
@@ -570,38 +741,44 @@ class PdfMetadata(MutableMapping):
                 for child in node.findall('*'):
                     node.remove(child)
                 if str(self._qname(key)) in LANG_ALTS:
-                    add_array(node, AltList([clean(val)]))
+                    add_array(node, AltList([_clean(val)]))
                 else:
-                    node.text = clean(val)
+                    node.text = _clean(val)
             else:
-                raise TypeError(val)
+                raise TypeError(f"Setting {key} to {val} with type {type(val)}")
         except StopIteration:
             # Insert a new node
             rdf = self._get_rdf_root()
             if str(self._qname(key)) in LANG_ALTS:
-                val = AltList([clean(val)])
+                val = AltList([_clean(val)])
             if isinstance(val, (list, set)):
                 rdfdesc = etree.SubElement(
                     rdf,
-                    QName(XMP_NS_RDF, 'Description'),
-                    attrib={QName(XMP_NS_RDF, 'about'): ''},
+                    str(QName(XMP_NS_RDF, 'Description')),
+                    attrib={str(QName(XMP_NS_RDF, 'about')): ''},
                 )
                 node = etree.SubElement(rdfdesc, self._qname(key))
                 add_array(node, val)
             elif isinstance(val, str):
-                rdfdesc = etree.SubElement(
+                _rdfdesc = etree.SubElement(  # lgtm [py/unused-local-variable]
                     rdf,
-                    QName(XMP_NS_RDF, 'Description'),
+                    str(QName(XMP_NS_RDF, 'Description')),
                     attrib={
                         QName(XMP_NS_RDF, 'about'): '',
-                        self._qname(key): clean(val),
+                        self._qname(key): _clean(val),
                     },
                 )
             else:
-                raise TypeError(val)
+                raise TypeError(
+                    f"Setting {key} to {val} with type {type(val)}"
+                ) from None
 
     @ensure_loaded
-    def __delitem__(self, key):
+    def __setitem__(self, key: Union[str, QName], val: Union[Set[str], List[str], str]):
+        return self._setitem(key, val, False)
+
+    @ensure_loaded
+    def __delitem__(self, key: Union[str, QName]):
         if not self._updating:
             raise RuntimeError("Metadata not opened for editing, use with block")
         try:
@@ -618,11 +795,11 @@ class PdfMetadata(MutableMapping):
             else:
                 parent.remove(node)
         except StopIteration:
-            raise KeyError(key)
+            raise KeyError(key) from None
 
     @property
     @ensure_loaded
-    def pdfa_status(self):
+    def pdfa_status(self) -> str:
         """Returns the PDF/A conformance level claimed by this PDF, or False
 
         A PDF may claim to PDF/A compliant without this being true. Use an
@@ -643,7 +820,7 @@ class PdfMetadata(MutableMapping):
 
     @property
     @ensure_loaded
-    def pdfx_status(self):
+    def pdfx_status(self) -> str:
         """Returns the PDF/X conformance level claimed by this PDF, or False
 
         A PDF may claim to PDF/X compliant without this being true. Use an
